"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const chai = require("chai");
const chaiAsPromised = require("chai-as-promised");
const typescript_service_helpers_1 = require("javascript-typescript-langserver/lib/test/typescript-service-helpers");
const fast_json_patch_1 = require("fast-json-patch");
chai.use(chaiAsPromised);
const assert = chai.assert;
function describeTypeScriptService(createService, shutdownService = typescript_service_helpers_1.shutdownTypeScriptService, rootUri) {
    describe('Workspace without project files', () => {
        beforeEach(typescript_service_helpers_1.initializeTypeScriptService(createService, rootUri, new Map([
            [rootUri + 'a.ts', 'const abc = 1; console.log(abc);'],
            [rootUri + 'foo/b.ts', ['/* This is class Foo */', 'export class Foo {}'].join('\n')],
            [rootUri + 'foo/c.ts', 'import {Foo} from "./b";'],
            [rootUri + 'd.ts', ['export interface I {', '  target: string;', '}'].join('\n')],
            [
                rootUri + 'local_callback.ts',
                'function local(): void { function act(handle: () => void): void { handle() } }',
            ],
            [
                rootUri + 'e.ts',
                [
                    'import * as d from "./d";',
                    '',
                    'let i: d.I = { target: "hi" };',
                    'let target = i.target;',
                ].join('\n'),
            ],
            [rootUri + 'foo/f.ts', ['import {Foo} from "./b";', '', 'let foo: Foo = Object({});'].join('\n')],
            [rootUri + 'foo/g.ts', ['class Foo = {}', '', 'let foo: Foo = Object({});'].join('\n')],
        ])));
        afterEach(shutdownService);
        describe('textDocumentXdefinition()', () => {
            specify('on interface field reference', function () {
                return __awaiter(this, void 0, void 0, function* () {
                    const result = yield this.service
                        .textDocumentXdefinition({
                        textDocument: {
                            uri: rootUri + 'e.ts',
                        },
                        position: {
                            line: 3,
                            character: 15,
                        },
                    })
                        .reduce(fast_json_patch_1.applyReducer, null)
                        .toPromise();
                    assert.deepEqual(result, [
                        {
                            location: {
                                uri: rootUri + 'd.ts',
                                range: {
                                    start: {
                                        line: 1,
                                        character: 2,
                                    },
                                    end: {
                                        line: 1,
                                        character: 8,
                                    },
                                },
                            },
                            symbol: {
                                filePath: 'd.ts',
                                containerName: 'd.I',
                                containerKind: '',
                                kind: 'property',
                                name: 'target',
                            },
                        },
                    ]);
                });
            });
            specify('in same file', function () {
                return __awaiter(this, void 0, void 0, function* () {
                    const result = yield this.service
                        .textDocumentXdefinition({
                        textDocument: {
                            uri: rootUri + 'a.ts',
                        },
                        position: {
                            line: 0,
                            character: 29,
                        },
                    })
                        .reduce(fast_json_patch_1.applyReducer, null)
                        .toPromise();
                    assert.deepEqual(result, [
                        {
                            location: {
                                uri: rootUri + 'a.ts',
                                range: {
                                    start: {
                                        line: 0,
                                        character: 6,
                                    },
                                    end: {
                                        line: 0,
                                        character: 9,
                                    },
                                },
                            },
                            symbol: {
                                filePath: 'a.ts',
                                containerName: '"a"',
                                containerKind: 'module',
                                kind: 'const',
                                name: 'abc',
                            },
                        },
                    ]);
                });
            });
        });
    });
}
exports.describeTypeScriptService = describeTypeScriptService;
//# sourceMappingURL=extended-typescript-service-helpers.js.map